@@ -33,17 +33,22 @@ import json
 import warnings
 import requests
 import ssl
+import copy
+import pprint
 
 from . import version
 
 from requests.adapters import HTTPAdapter
+from six.moves.urllib.parse import urlparse
 
 try:
     from requests.packages.urllib3.poolmanager import PoolManager
     from requests.packages.urllib3 import exceptions
+    from requests.packages.urllib3.util.retry import Retry
 except ImportError:
     from urllib3.poolmanager import PoolManager
     from urllib3 import exceptions
+    from urllib3.util.retry import Retry
 
 from databricks_cli.version import version as databricks_cli_version
 
@@ -67,10 +72,21 @@ class ApiClient(object):
         if host[-1] == "/":
             host = host[:-1]
 
+        retries = Retry(
+            total=6,
+            backoff_factor=1,
+            status_forcelist=[429],
+            method_whitelist=set({'POST'}) | set(Retry.DEFAULT_METHOD_WHITELIST),
+            respect_retry_after_header=True,
+            raise_on_status=False # return original response when retries have been exhausted
+        )
         self.session = requests.Session()
-        self.session.mount('https://', TlsV1HttpAdapter())
+        self.session.mount('https://', TlsV1HttpAdapter(max_retries=retries))
 
-        self.url = "%s/api/%s" % (host, apiVersion)
+        parsed_url = urlparse(host)
+        scheme = parsed_url.scheme
+        hostname = parsed_url.hostname
+        self.url = "%s://%s/api/%s" % (scheme, hostname, apiVersion)
         if user is not None and password is not None:
             encoded_auth = (user + ":" + password).encode()
             user_header_data = "Basic " + base64.standard_b64encode(encoded_auth).decode()
@@ -97,14 +113,38 @@ class ApiClient(object):
         """set up connection and perform query"""
         if headers is None:
             headers = self.default_headers
+        else:
+            tmp_headers = copy.deepcopy(self.default_headers)
+            tmp_headers.update(headers)
+            headers = tmp_headers
 
         with warnings.catch_warnings():
             warnings.simplefilter("ignore", exceptions.InsecureRequestWarning)
-            resp = self.session.request(method, self.url + path, data = json.dumps(data),
-                verify = self.verify, headers = headers)
-
+            if method == 'GET':
+                translated_data = {k: _translate_boolean_to_query_param(data[k]) for k in data}
+                resp = self.session.request(method, self.url + path, params = translated_data,
+                                            verify = self.verify, headers = headers)
+            else:
+                resp = self.session.request(method, self.url + path, data = json.dumps(data),
+                                            verify = self.verify, headers = headers)
         try:
             resp.raise_for_status()
         except requests.exceptions.HTTPError as e:
-            raise e
+            message = e.args[0]
+            try:
+                reason = pprint.pformat(json.loads(resp.text), indent=2)
+                message += '\n Response from server: \n {}'.format(reason)
+            except ValueError:
+                pass
+            raise requests.exceptions.HTTPError(message, response=e.response)
         return resp.json()
+
+
+def _translate_boolean_to_query_param(value):
+    assert not isinstance(value, list), 'GET parameters cannot pass list of objects'
+    if isinstance(value, bool):
+        if value:
+            return 'true'
+        else:
+            return 'false'
+    return value
